// --------------------------------------------------------------------------------
// App
// --------------------------------------------------------------------------------

if ('addEventListener' in document) {
	document.addEventListener('DOMContentLoaded', function() {
		FastClick.attach(document.body);
	}, false);
}

// Shim until we activate MixPanel
window.mixpanel = window.mixpanel || {
	track: function(){}
};


// Set jQuery mobile options
$(document).bind("mobileinit", function() {
	// Setup jqm
	$.extend($.mobile, {
		defaultPageTransition : 'fade',
		touchOverflowEnabled  : true,
		allowCrossDomainPages : true
	});
});

function refresh() {
	$('.loading').show().css('opacity', '1');
	Trends.getTrends(function(data) {
		console.log(data);
		if(data.error) {
			if(data.error == "no games") {
				show_modal("There are no games scheduled today.");
			}
			if(data.error == "still mining") {
				show_modal("Sorry! The servers are currently processing trends. Check back soon!");
			}
			if(data.error == "no compelling") {
				show_modal("No games had trends compelling enough to be featured in the app. Check back tomorrow!");
			}

			show_no_games();
		} else if(data.fail) {
			if(data.fail == 'Unauthorized') {
				var msg = "<p>There is nothing here!</p>"
					+ "<p>The " + Config.league.toUpperCase() + " season is now finished and so are this year's trends.</p>"
					+ "<p>Check back at the start of next season for the new app!</p>";
				show_modal(msg);
			} else if(data.fail == 'Date too old' || data.fail == 'Date too new') {
				show_modal("Please ensure that the date/time is correctly set on your device.");
			} else {
				show_modal("An unexpected error occurred while trying to fetch trends.");
			}
			show_no_games();
		} else {
			// Render views, bind data to li nodes
			$('#wl').html(tmpl("wl_tmpl",   data)).listview("refresh").bindData(data.wl, 'wl');   // Win / Loss
			$('#ou').html(tmpl("ou_tmpl",   data)).listview("refresh").bindData(data.ou, 'ou');   // Over / Under
			$('#roi').html(tmpl("roi_tmpl", data)).listview("refresh").bindData(data.roi, 'roi'); // ROI
			$('#ats').html(tmpl("ats_tmpl", data)).listview("refresh").bindData(data.ats, 'ats'); // ATS

			// Do we have a sponsor?
			$('#game-details').data('sponsor', data.sponsor || null);

			// Remove loading screen...
			$('.loading').fadeTo(500, 0, function() { $(this).hide(); });

			window.DATA_LOADED = true;
		}
	});
};

var check_connectivity = function() {
	var net = navigator.connection.type != Connection.NONE;
	if(net == window.HAS_INTERNET) {
		// No change in connectivity.
		return;
	}
	window.HAS_INTERNET = net;

	// We already have the data - don't care about connectivity changes.
	if(window.DATA_LOADED) return;

	if(window.HAS_INTERNET) {
		hide_modal();
		refresh();
	} else {
		show_modal("This application requires an Internet connection.");
	}
};

var init = function() {
	if(window.NATIVE) {
		check_connectivity();
		setInterval(check_connectivity, 1000);
	}

	// Setup app title
	var title = Config.title;
	if (Config.year) title += ' <em>'+ Config.year +'</em>';
	$('div[data-role="header"] h1').html(title);

	// Hide status bar
	if (window.StatusBar) {
		StatusBar.hide();
	}

	// Modal setup
	$('#modal button').click(function(){
		$('#modal, .modal-backdrop').hide();
		$('#modal .modal-body').html('');
	});

	// Tabs description
	$('.home-tabs a').bind('tap click', function() {
		var desc = $(this).attr('data-description');
		$('.home-tabs .ui-title').html(desc);
	});

	// Hide ATS tab for baseball
	if (Config.league == 'mlb') {
		$('.tab-ats').hide();
	}

	// FAQ
	$('.custom-help').bind('tap', function() {
		mixpanel.track("FAQ viewed");

		var url = Config.apiHost +'/faq/'+ Config.league;
		var tgt = '_blank';
		if(window.device && device.platform == 'Android') {
			// There is an issue where the inappbrowser window does not properly
			// close - witnessed on Lollipop.
			tgt = '_system';
		}
		window.open(url, tgt);
	});

	refresh();

	// Populate results page
	$('#wl li, #ou li, #roi li, #ats li').live('tap', function() {
		var data = $(this).data('event');

		// If we don't have an event key, then we're probably heading to
		// a UI view that doesn't involve trend results.
		if(data.event_key == "") return;

		switch(data.event_type) {
			case 'wl':  mixpanel.track("W/L Trend Viewed"); break;
			case 'ou':  mixpanel.track("O/U Trend Viewed"); break;
			case 'ats': mixpanel.track("ATS Trend Viewed"); break;
			case 'roi': mixpanel.track("ROI Trend Viewed"); break;
		}

		// format data...
		data.bet_outcome = (data.type == "wins" || $.inArray(data.event_type, ['ou', 'roi', 'ats']) != -1) ? 'bet' : 'fade'; // bet, fade
		data.bet_title   = (data.team == data.matchup.at_name) ? data.matchup.at_fname : data.matchup.ht_fname;

		// normalize date so JS can figure it out
		var jsdate = data.matchup.start_time.replace(/T/g, ' ').replace(/-/gi, '/') + ' UTC'; + ' UTC';
		var d = new Date(Date.parse(jsdate));
		var hour = d.getHours();
		var minutes = d.getMinutes();
		var ampm = hour > 11 ? "PM" : "AM";
		if(hour > 12) hour = hour - 12;
		if(hour == 0) hour = 12;
		if(minutes < 10) minutes = '0' + d.getMinutes();

		data.start_time = hour + ':' + minutes + ' ' + ampm;
		data.start_date = dateFormat(d, 'dddd, mmmm dS @ h:MM TT');

		var toFixed = function(v, addplus) {
			v = parseFloat(v);
			if(isNaN(v)) return '';
			v = v.toFixed(1);
			if(addplus && v > 0) return '+' + v;
			return v;
		}

		if(data.lines) {
			if(data.lines.total && data.lines.total.over) {
				data.lines.total.over.value = toFixed(data.lines.total.over.value);
				data.lines.total.under.value = toFixed(data.lines.total.under.value);
			}
			if(data.lines.spread && data.lines.spread.home) {
				data.lines.spread.home.value = toFixed(data.lines.spread.home.value, true);
				data.lines.spread.away.value = toFixed(data.lines.spread.away.value, true);
			}
		}

		// stats data changes based on event type
		if (data.event_type == "wl") {
			var wl = (data.type == "wins" ? 'won' : 'lost');
			data.intro = 'The '+ data.team +' have '+ wl +' <span class="outcome">'+ data.score + '%</span>'
				+ ' of games in this situation:';
			data.stats_eng = 'W-L : ' + data.wins + '-' + data.losses
				+ ' (<span class="outcome">'+ ucFirst(wl) + ' ' + data.score
				+ '%</span> <small>SINCE 2006</small>)';
			data.bet_eng = 'You should <span class="outcome">'
				+ (wl == "won" ? '' : 'not ') + 'bet</span> '
				+ ucFirst(data.bet_title)
				+ ' to win their next game!';
		}
		if (data.event_type == "ats") {
			var t = data.type == 'covered' ? 'to cover' : 'to not cover';
			var pt = data.type == 'covered' ? 'have covered'  : 'did not cover';
			data.intro = 'The '+ data.team +' '+ pt +' <span class="outcome">'+ data.score +'%</span>'
				+ ' of games in this situation:';
			data.bet_title = data.type == 'didnotcover' ? 'Against the Spread' : 'the Spread';
			data.stats_eng = 'ATS: '+ data.covered +' / ' + data.didnotcover
				+ ' (<span class="outcome">' + ucFirst(data.type == 'covered' ? 'covered' : 'Didn\'t Cover') +' '+ data.score
				+ '%</span> <small>SINCE 2006</small>)';
			data.bet_eng = 'Bet '+ data.team + ' <span class="outcome">' + t +' the spread</span>'
				+ ' in their next game';
		}
		if (data.event_type == "ou") {
			data.intro = 'Games have gone '+ data.type +' the total <span class="outcome">'
				+ data.score +'%</span> of the time in this situation:';
			data.bet_title = ucFirst(data.type);
			data.stats_eng = 'O-U: '+ data.overs +' / ' + data.unders
				+ ' (<span class="outcome">' + ucFirst(data.type) + ' '+ data.score
				+ '%</span> <small>SINCE 2006</small>)';
			data.bet_eng = 'Bet the game to go <span class="outcome">'
				+ data.type +'</span> the Total!';
		}
		if(data.event_type == "roi") {
			data.intro = '@todo ROI intro';
			data.stats_eng = 'W-L : ' + data.wins + '-' + data.losses
				+ ' (ROI: <span class="outcome">' + data.score
				+ '%</span> <small>SINCE 2006</small>)';
			data.bet_eng = ucFirst(data.bet_title) + ' has a <span class="outcome">'
				+ data.score + '% ROI</span> in this situation';
		}

		// add class to reflect some colour changes for bet and fade
		$('.results').removeClass('fade bet').addClass(data.bet_outcome);

		// render views
		$('#intro').html(tmpl("intro_tmpl", data));
		$('#trend').html(tmpl("trend_tmpl", data));
		$('#stats').html(tmpl("stats_tmpl", data));
		$('#game-details').html(tmpl("game_details_tmpl", data));

		var sponsor = $('#game-details').data('sponsor');
		if(sponsor) {
			$('#sponsor').html(sponsor.html || "").show();
		}
	});
};

var deviceinit = function() {
	// are we native-wrapped?
	window.NATIVE = (typeof window.cordova != 'undefined');
	window.HAS_INTERNET = true;
	window.DATA_LOADED = false;
	window.CURRENT_DATE = get_date_pst();

	init();
};

var checkdate = function() {
	// If PST date has changed since we last looked, then initiate a data refresh.
	var now = get_date_pst();
	if(now.getDate() == window.CURRENT_DATE.getDate()) {
		return;
	}
	window.CURRENT_DATE = now;
	refresh();
};

if(window.cordova) {
	document.addEventListener('deviceready', deviceinit, false);
	document.addEventListener('resume', checkdate, false);
} else {
	$(deviceinit);
}

// --------------------------------------------------------------------------------
// Utilities and Plugins
// --------------------------------------------------------------------------------

// Get the current date relative to the provided UTC offset (in seconds).
function get_date(utc_offset) {
	var now = new Date(Date.now());
	var utc = Date.UTC(now.getUTCFullYear(), now.getUTCMonth(), now.getUTCDate());
	return new Date(utc - (utc_offset * 1000));
}

// Get the current date in Pacific Standard Time.
function get_date_pst() {
	return get_date(-28800);
}

function show_no_games() {
	// Error
	$('#wl, #ou, #ats, #roi').html('<li class="no-games"><span class="team">No games to show</span></li>');
}

function show_modal(msg) {
	$('.loading').hide();
	$('#modal .modal-body').html(msg);
	$('#modal').show();
}

function hide_modal() {
	$('#modal').hide();
}

function ucFirst(string) {
	var words = string.split(' ');
	for(var i=0; i<words.length; i++) {
		words[i] = words[i].substring(0, 1).toUpperCase() + words[i].substring(1).toLowerCase();
	}
	return words.join(' ');
};

/**
 * jQuery Plugin by Jono
 * - Binds data to to the li elements
 */
(function($){
	$.fn.bindData = function(data, event_type) {
		$.each(this.find('li'), function(i , $o) {
			if(typeof(data[i]) == "undefined") return;
			data[i].event_type = event_type;
			$(this).data('event', data[i]);
		});
	};
})(jQuery);
